-- Copyright 2008 Steven Barth <steven@midlink.org>
-- Copyright 2011 Jo-Philipp Wich <jow@openwrt.org>
-- Licensed to the public under the Apache License 2.0.

module("luci.controller.admin.status", package.seeall)

function index()
	entry({"admin", "status"}, alias("admin", "status", "overview"), _("Status"), 20).index = true
	entry({"admin", "status", "overview"}, call("action_index"), _("Overview"), 1)
	entry({"admin", "status", "wifi"}, template("admin_status/wifi"), _("Wifi"), 2)
	entry({"admin", "status", "eth_setting"}, template("admin_status/eth_setting"), _("Eth IP Setting"), 3)
  entry({"admin", "status", "wifi_setting"}, template("admin_status/wifi_setting"), _("WiFi IP Setting"), 4)
end

function action_syslog()
	local syslog = luci.sys.syslog()
	luci.template.render("admin_status/syslog", {syslog=syslog})
end

function action_dmesg()
	local dmesg = luci.sys.dmesg()
	luci.template.render("admin_status/dmesg", {dmesg=dmesg})
end

function action_index()
	local patch_check="true"
	local sys = require "luci.sys"
        local fs  = require "nixio.fs"
        local restore_cmd = "tar -xzC/ >/dev/null 2>&1"
        local backup_cmd  = "sysupgrade --create-backup - 2>/dev/null"
        local image_helium_tmp   = "/tmp/helium_gateway.ipk"
	 local update_patch_tmp   = "/tmp/patch.tar.gz"
	local patch_path
	local filname
	local res;
	local update_res
	local function patch_checksum()
		return (luci.sys.exec("sha256sum %q" % patch_path):match("^([^%s]+)"))
	end
	local function firmwareupdate()
		return (luci.sys.exec("/update/manualpatch.sh %q" % filname):match("^([^%s]+)"))
	end
        local fp
        luci.http.setfilehandler(
                function(meta, chunk, eof)
                        if not fp then
                                if meta and meta.name == "update_helium" then
                                        fp = io.open(image_helium_tmp, "w")
                                elseif meta and meta.name == "update_firmware" then
						patch_path = "/tmp/" .. meta.file
						filname = meta.file
						fp = io.open(patch_path, "w")
                                else
                                        fp = io.popen(restore_cmd, "w")
                                end
                        end
                        if chunk then
                                fp:write(chunk)
                        end
                        if eof then
                                fp:close()
                        end
                end
        )
	local check = luci.http.formvalue("patch_checksum")
        if luci.http.formvalue("update_helium") then
		luci.sys.exec("opkg install /tmp/helium_gateway.ipk")
		luci.sys.exec("sleep 1;cp /update/install_update /etc/helium_gateway/install_update;sync;sleep 1;/etc/init.d/helium_gateway restart")
		luci.template.render("admin_status/index")
        elseif luci.http.formvalue("update_firmware") then
		local checksum1=patch_checksum()
		if checksum1 == check then
			patch_check="true"
			res=firmwareupdate()
			if res == "case1" then
				update_res = "firmware updated OK"
			elseif res == "case2" then
				update_res = "firmware updated failed"
			elseif res == "case3" then
				update_res = "the patch maybe is not used for HT-M7603 or not from heltec"
			else
				update_res = "firmware updated failed,it has auto updated to" .. res
			end		
		else
			patch_check="false"
		end
		luci.template.render("admin_status/index",{patch_path=patch_path,checksum1=checksum1,patch_check=patch_check,update_res=update_res})
         else
                luci.template.render("admin_status/index",{patch_check=patch_check})
        end
end

function action_iptables()
	if luci.http.formvalue("zero") then
		if luci.http.formvalue("zero") == "6" then
			luci.util.exec("ip6tables -Z")
		else
			luci.util.exec("iptables -Z")
		end
		luci.http.redirect(
			luci.dispatcher.build_url("admin", "status", "iptables")
		)
	elseif luci.http.formvalue("restart") == "1" then
		luci.util.exec("/etc/init.d/firewall reload")
		luci.http.redirect(
			luci.dispatcher.build_url("admin", "status", "iptables")
		)
	else
		luci.template.render("admin_status/iptables")
	end
end

function action_bandwidth(iface)
	luci.http.prepare_content("application/json")

	local bwc = io.popen("luci-bwc -i %q 2>/dev/null" % iface)
	if bwc then
		luci.http.write("[")

		while true do
			local ln = bwc:read("*l")
			if not ln then break end
			luci.http.write(ln)
		end

		luci.http.write("]")
		bwc:close()
	end
end

function action_wireless(iface)
	luci.http.prepare_content("application/json")

	local bwc = io.popen("luci-bwc -r %q 2>/dev/null" % iface)
	if bwc then
		luci.http.write("[")

		while true do
			local ln = bwc:read("*l")
			if not ln then break end
			luci.http.write(ln)
		end

		luci.http.write("]")
		bwc:close()
	end
end

function action_load()
	luci.http.prepare_content("application/json")

	local bwc = io.popen("luci-bwc -l 2>/dev/null")
	if bwc then
		luci.http.write("[")

		while true do
			local ln = bwc:read("*l")
			if not ln then break end
			luci.http.write(ln)
		end

		luci.http.write("]")
		bwc:close()
	end
end

function action_connections()
	local sys = require "luci.sys"

	luci.http.prepare_content("application/json")

	luci.http.write("{ connections: ")
	luci.http.write_json(sys.net.conntrack())

	local bwc = io.popen("luci-bwc -c 2>/dev/null")
	if bwc then
		luci.http.write(", statistics: [")

		while true do
			local ln = bwc:read("*l")
			if not ln then break end
			luci.http.write(ln)
		end

		luci.http.write("]")
		bwc:close()
	end

	luci.http.write(" }")
end

function action_nameinfo(...)
	local i
	local rv = { }
	for i = 1, select('#', ...) do
		local addr = select(i, ...)
		local fqdn = nixio.getnameinfo(addr)
		rv[addr] = fqdn or (addr:match(":") and "[%s]" % addr or addr)
	end

	luci.http.prepare_content("application/json")
	luci.http.write_json(rv)
end
